% backproject_estimate.m
% Austin Nicolai
% 10/12/16
%
% purpose:
%
%   takes coordinates in pixel space returned from select_features.m and
%   backprojects them into 3D space for use in factor graph reconstruction.
%   the unknown elevation angle is naively assume to be zero, resulting
%   in points in the zero plane in 3D space.
%
% outputs:
%
%   pts_euclid: features points in 3D space.
%
% inputs:
%
%   img_dim:   the dimentions of the sonar image, in pixels.
%   x_polar:   the x-locations of the features, in sonar image pixel
%              coordinates.
%   y_polar:   the y-locations of the features, in sonar image pixel
%              coordinates.
%   max_range: the max range of the sonar image (in meters), used for
%              scaling purposes.

function [pts_euclid] = backproject_estimate(img_dim, x_polar, y_polar, max_range)%, xform_shift, xform_rot)

%validate input
assert(isequal(size(img_dim), [1 2]), 'img_dim argument must be of size [1 2]');

%inits
img_h = img_dim(1);
img_w = img_dim(2);
x_offset = img_w/2;

num_features = size(x_polar, 1);
num_frames = size(x_polar, 2);

r = zeros(num_features, num_frames);
psi = zeros(num_features, num_frames);

%compute polar coordinates of features
%   y values must be flipped since matlab indexes from top
%   x values must be offset since sonar center is center of image

x_new = x_polar - x_offset;
y_new = abs(y_polar - img_h);

for i = 1:num_frames
    for j = 1:num_features
        
        psi(j,i) = atan2d(y_new(j,i),x_new(j,i));
        r(j,i)   = sqrt( x_new(j,i)^2 + y_new(j,i)^2 ); % in pixels
        
    end
end

%convert r from pixel space to real space
%scale images by the max range

r_scale  = img_h / max_range; % in meters
r_new = r / r_scale;

% compute arcs for visualization.
min_theta = -10; % using +/- 10 degrees for 20 degree FOV
max_theta = 10;
num_thetas = 1;
theta = 0; % set pitch to 0 for naive guess

pts_euclid = zeros(num_frames, num_features, num_thetas, 3);

for i = 1:num_frames
    for j = 1:num_features
        for k = 1:num_thetas
            
            pts_euclid(i, j, k, 1) = r_new(j,i)*cosd(psi(j,i))*cosd(theta(1,k));
            pts_euclid(i, j, k, 2) = r_new(j,i)*sind(psi(j,i))*cosd(theta(1,k));
            pts_euclid(i, j, k, 3) = r_new(j,i)*sind(theta(1,k));
            
        end
    end
end

end
