/**
 * @file example.cpp
 * @brief Simple iSAM example with dynamic allocation.
 * @author Michael Kaess
 * @version $Id: example.cpp 6377 2012-03-30 20:06:44Z kaess $
 *
 * Copyright (C) 2009-2013 Massachusetts Institute of Technology.
 * Michael Kaess, Hordur Johannsson, David Rosen,
 * Nicholas Carlevaris-Bianco and John. J. Leonard
 *
 * This file is part of iSAM.
 *
 * iSAM is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * iSAM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with iSAM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/** @page Example Simple iSAM Example

The iSAM library represents general optimization problems as a factor graph, where nodes represent variables and factors represent measurements on sets of one or more nodes.

In this simple SLAM example, five poses are created and connected into a chain by odometry measurements. A landmark is added with a single measurement from one of the poses.

Note that the optimization problem described so far is not fully constrained as the complete graph can be shifted and rotated arbitrarily in the plane without changing the relative measurements described by odometry and landmark measurements. The graph needs to be constraints, and a common solution is to add a prior to the first pose to fix it at the origin.

The following code can be compiled and run with
@verbatim
make examples
bin/example
@endverbatim

To compile without the iSAM cmake system:
@verbatim
cd examples
c++ example.cpp ../lib/libisam.a -lcholmod -I../include -I/usr/include/eigen3
@endverbatim

examples/example.cpp :
@includelineno examples/example.cpp

*/


/**
 * This example has been built off of the simple example provided with
 * the iSAM code. Note that below, this can be more efficiently implemented
 * in a loop structure, reading in data from a file. However, in this
 * template, all nodes are added explicitly for illustration purposes.
 */

#include <isam/isam.h>

using namespace std;
using namespace isam;
using namespace Eigen;

int main() {
    
  // instance of the main class that manages and optimizes the pose graph.
  Slam slam;

  // create noise vectors as necessary for the nav and feature observation
  // measurements.
  Noise noise6d = Information(1 * eye(6));
  Noise noise3d = Information(1 * eye(3));

  // create and add all poses of the vehicle to the factor graph in the
  // form: (x, y, z, yaw, pitch, roll).
  Pose3d_Node* pose_node_1 = new Pose3d_Node();
  Pose3d_Node* pose_node_2 = new Pose3d_Node();
  Pose3d_Node* pose_node_N = new Pose3d_Node();

  slam.add_node(pose_node_1);
  slam.add_node(pose_node_2);
  slam.add_node(pose_node_N);

  // add anchor node. currently setting the anchor at the origin.
  Pose3d origin(0., 0., 0., 0., 0., 0.);
  Pose3d_Factor* anchor = new Pose3d_Factor(pose_node_1, origin, noise6d);
  slam.add_factor(anchor);

  // create and add all nav odometry measurements to the factor graph, in
  // radians. using the notation of "odometry_startpose_endpose".
  Pose3d odometry_1_2(0., 0., 0., 0., 0., 0.);
  Pose3d odometry_2_N(0., 0., 0., 0., 0., 0.);

  Pose3d_Pose3d_Factor* constraint_1 = new Pose3d_Pose3d_Factor(pose_node_1, pose_node_2, odometry_1_2, noise6d);
  Pose3d_Pose3d_Factor* constraint_2 = new Pose3d_Pose3d_Factor(pose_node_2, pose_node_N, odometry_2_N, noise6d);

  slam.add_factor(constraint_1);
  slam.add_factor(constraint_2);

  // create and add all landmarks to the factor graph, in the form of
  // (x, y, z).
  Point3d_Node* point_node_1 = new Point3d_Node();
  Point3d_Node* point_node_2 = new Point3d_Node();
  Point3d_Node* point_node_N = new Point3d_Node();

  slam.add_node(point_node_1);
  slam.add_node(point_node_2);
  slam.add_node(point_node_N);

  // create and add all landmark measurements to the factor graph. using
  // the notation of "measure_viewpose_landmark". currently assuming all
  // landmarks are visible from all poses.
  Point3d measure_1_1(0., 0., 0.);
  Point3d measure_1_2(0., 0., 0.);
  Point3d measure_1_N(0., 0., 0.);

  Point3d measure_2_1(0., 0., 0.);
  Point3d measure_2_2(0., 0., 0.);
  Point3d measure_2_N(0., 0., 0.);

  Point3d measure_N_1(0., 0., 0.);
  Point3d measure_N_2(0., 0., 0.);
  Point3d measure_N_N(0., 0., 0.);

  Pose3d_Point3d_Factor* measurement_01 = new Pose3d_Point3d_Factor(pose_node_1, point_node_1, measure_1_1, noise3d);
  Pose3d_Point3d_Factor* measurement_02 = new Pose3d_Point3d_Factor(pose_node_1, point_node_2, measure_1_2, noise3d);
  Pose3d_Point3d_Factor* measurement_03 = new Pose3d_Point3d_Factor(pose_node_1, point_node_N, measure_1_N, noise3d);

  Pose3d_Point3d_Factor* measurement_04 = new Pose3d_Point3d_Factor(pose_node_2, point_node_1, measure_2_1, noise3d);
  Pose3d_Point3d_Factor* measurement_05 = new Pose3d_Point3d_Factor(pose_node_2, point_node_2, measure_2_2, noise3d);
  Pose3d_Point3d_Factor* measurement_06 = new Pose3d_Point3d_Factor(pose_node_2, point_node_N, measure_2_N, noise3d);

  Pose3d_Point3d_Factor* measurement_07 = new Pose3d_Point3d_Factor(pose_node_N, point_node_1, measure_3_1, noise3d);
  Pose3d_Point3d_Factor* measurement_08 = new Pose3d_Point3d_Factor(pose_node_N, point_node_2, measure_3_2, noise3d);
  Pose3d_Point3d_Factor* measurement_09 = new Pose3d_Point3d_Factor(pose_node_N, point_node_N, measure_3_N, noise3d);

  slam.add_factor(measurement_01);
  slam.add_factor(measurement_02);
  slam.add_factor(measurement_03);
  slam.add_factor(measurement_04);
  slam.add_factor(measurement_05);
  slam.add_factor(measurement_06);
  slam.add_factor(measurement_07);
  slam.add_factor(measurement_08);
  slam.add_factor(measurement_09);

  // set properties for the optimization as desired.
  cout << "Setting properties: " << endl;
  cout << "\tverbose = true" << endl;
  cout << "\tmethod = LEVENBERG_MARQUARDT" << endl;

  isam::Properties props;
  props.verbose = true;
  props.method = LEVENBERG_MARQUARDT;

  slam.set_properties(props);

  // optimize the factor graph.
  cout << "Optimizing the factor graph." << endl;
  cout << endl;
  cout << "Iterations to convergence: " << slam.batch_optimization() << endl;
  cout << endl;

  // print the final graph.
  cout << "Full graph: " << endl;
  slam.write(cout);
 
  // end program
  return 0;
}
