% select_features.m
% Austin Nicolai
% 10/8/16
%
% purpose:
%
%   manually select feature points from processed sonar frames. selected
%   feature points are displayed on the image, and the feature coordinates
%   are returned in pixel space. checks for appropriate display scaling
%   and allows for the display colormap to be selected. requires ginputc.m
%   file from MATLAB file exchange to adjust ginput crosshair colors.
%
%   only tested for use with the output of process_csv.m.
%
% outputs:
%
%   x: the x-locations of the features, selected by the user.
%   y: the y-locations of the features, selected by the user.
%
% inputs:
%
%   img:          array of all processed sonar frames.
%   num_features: the number of features to select per frame.
%   cmap:         the colormap to use.

function [x, y] = select_features(img, num_features, cmap)

% init
img_h = size(img, 1);
img_w = size(img, 2);
num_frames = size(img, 3);

x = zeros(num_features, num_frames);
y = zeros(num_features, num_frames);
marker_size = floor(img_w/60); % chosen experimentally

% for all frames
for curr_frame = 1:num_frames
    
    % reshape to 2D for display
    img_view = img(:, :, curr_frame);
    img_view = reshape(img_view, [img_h, img_w]);
    
    if isa(img_view, 'float') && (max(max(img_view)) >= 1)
        imshow(flipud(img_view/256), 'ColorMap', colormap(cmap)); % scale to [0,1] since float
    else
        imshow(flipud(img_view), 'ColorMap', colormap(cmap));
    end
    
    % get mouse input, and display click locations
    hold on;
    for curr_feat = 1:num_features
        
        % get next feature location
        [x(curr_feat, curr_frame), y(curr_feat, curr_frame)] = ginputc(1, 'Color', 'g'); % uses ginputc.m to change crosshair color
        
        % show the clicked location
        plot(x(curr_feat, curr_frame), y(curr_feat, curr_frame), 'Marker', 'o', ...
            'MarkerEdgeColor', 'g', 'MarkerFaceColor', 'g', 'MarkerSize', marker_size);
        
    end
    
    % extra click for a "pause" between frames. do nothing with this input.
    ginputc(1, 'Color', 'r'); % uses ginputc.m to change crosshair color
    hold off;
    
end

% close the figures
close all;

end
